/** ADAMEm: Coleco ADAM emulator ********************************************/
/**                                                                        **/
/**                               convdisk.c                               **/
/**                                                                        **/
/** This program converts disk images from non-standard disk types to      **/
/** standard ones or tape images                                           **/
/**                                                                        **/
/** Copyright (C) Marcel de Kogel 1996,1997                                **/
/**     You are not allowed to distribute this software commercially       **/
/**     Please, notify me, if you make any changes to this file            **/
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

static int src_sectors=8,src_interleave=5;
static int dest_sectors=8,dest_interleave=5;
static int src_interleave_table[128];
static int dest_interleave_table[128];
static char buf[512];

static char *Options[]= { "s","i","t","d","ds","di",NULL };

int main (int argc,char *argv[])
{
 FILE *infile,*outfile;
 char *in=NULL,*out=NULL;
 int i,j,n;
 int error=0,misparm=0,len;
 printf ("convdisk: ADAM disk image converter\n"
         "Copyright (C) Marcel de Kogel 1996,1997\n");
 for(n=1,i=0;n<argc;n++)
 {
  if(*argv[n]!='-')
   switch(i++)
   {
    case 0:  in=argv[n];
             break;
    case 1:  out=argv[n];
             break;
    default: misparm=1;
             break;
   }
  else
  {    
   for(j=0;Options[j];j++)
    if(!strcmp(argv[n]+1,Options[j])) break;
   switch(j)
   {
    case 0:  n++;
             if(n<argc)
              src_sectors=atoi(argv[n]);
             else
              misparm=1;
             break;
    case 1:  n++;
             if(n<argc)
              src_interleave=atoi(argv[n]);
             else
              misparm=1;
             break;
    case 2:  dest_interleave=1;
             dest_sectors=8;
             break;
    case 3:  dest_interleave=5;
             dest_sectors=8;
             break;
    case 4:  n++;
             if(n<argc)
              dest_sectors=atoi(argv[n]);
             else
              misparm=1;
             break;
    case 5:  n++;
             if(n<argc)
              dest_interleave=atoi(argv[n]);
             else
              misparm=1;
             break;
    default: misparm=1;
             break;
   }
  }
 }
 if (misparm || !in || !out)
 {
  printf ("Usage: convdisk [options] <source> <destination>\n"
          "Available options are:\n"
          "-s <value>  - Specify number of sectors per track of source [8]\n"
          "-i <value>  - Specify sector interleave of source [5]\n"
          "-ds <value> - Specify number of sectors per track of destination [8]\n"
          "-di <value> - Specify sector interleave of destination [5]\n"
          "-t          - Output a tape image (same as -ds 8 -di 1)\n"
          "-d          - Output a disk image (same as -ds 8 -di 5)\n");
  return 1;
 }
 printf ("Source has %d sectors per track, interleave is %d:1\n"
         "Destination has %d sectors per track, interleave is %d:1\n",
         src_sectors,src_interleave,dest_sectors,dest_interleave);
 j=0;
 for (i=0;i<src_sectors;++i)
 {
  src_interleave_table[i]=j;
  j+=src_interleave;
  do
  {
   j%=src_sectors;
   for (n=0;n<i;++n)
    if (src_interleave_table[n]==j) { n=100; j++; break; }
  }
  while (n>=100);
 }
 j=0;
 for (i=0;i<dest_sectors;++i)
 {
  dest_interleave_table[i]=j;
  j+=dest_interleave;
  do
  {
   j%=dest_sectors;
   for (n=0;n<i;++n)
    if (dest_interleave_table[n]==j) { n=100; j++; break; }
  }
  while (n>=100);
 }
 infile=fopen (in,"rb");
 if (!infile)
 {
  printf ("Cannot open %s\n",in);
  return 1;
 }
 fseek (infile,0,SEEK_END);
 len=ftell(infile)/512;
 fseek (infile,0,SEEK_SET);
 outfile=fopen (out,"wb");
 if (!outfile)
 {
  printf ("Cannot open %s\n",out);
  return 1;
 }
 for (i=0;i<len;++i)
 {
  j=((i/src_sectors)*src_sectors)+src_interleave_table[i%src_sectors];
  if (fseek(infile,j*512,SEEK_SET))
  {
   printf ("Error seeking sector %u in file %s\n",i,in);
   error=1;
   break;
  }
  if (!fread(buf,512,1,infile))
  {
   printf ("Error reading sector %u\n",i);
   error=1;
   break;
  }
  j=((i/dest_sectors)*dest_sectors)+dest_interleave_table[i%dest_sectors];
  if (fseek(outfile,j*512,SEEK_SET))
  {
   printf ("Error seeking sector %u in file %s\n",i,out);
   error=1;
   break;
  }
  if (!fwrite(buf,512,1,outfile))
  {
   printf ("Error writing sector %u\n",i);
   error=1;
   break;
  }
 }
 fclose (infile);
 fclose (outfile);
 printf ("%u sectors written\n",i);
 return error;
}
